@@ -14,6 +14,7 @@
 
 import asyncio
 import json
+import time
 from concurrent.futures.thread import ThreadPoolExecutor
 from typing import Any, TYPE_CHECKING, Optional
 
@@ -132,7 +133,10 @@ class IconScoreInnerTask(object):
     async def invoke(self, request: dict) -> dict:
         Logger.debug(tag=_TAG, msg=f'invoke() start')
 
-        self._check_icon_service_ready()
+        try:
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
 
         if self._is_thread_flag_on(EnableThreadFlag.INVOKE):
             loop = asyncio.get_event_loop()
@@ -168,13 +172,15 @@ class IconScoreInnerTask(object):
             converted_prev_block_validators = params.get('prevBlockValidators')
             converted_prev_votes = params.get('prevBlockVotes')
 
-            tx_results, state_root_hash, added_transactions, next_preps = self._icon_service_engine.invoke(
-                block=block,
-                tx_requests=converted_tx_requests,
-                prev_block_generator=converted_prev_block_generator,
-                prev_block_validators=converted_prev_block_validators,
-                prev_block_votes=converted_prev_votes,
-                is_block_editable=converted_is_block_editable)
+            tx_results, state_root_hash, added_transactions, next_preps, is_shutdown = \
+                self._icon_service_engine.invoke(
+                    block=block,
+                    tx_requests=converted_tx_requests,
+                    prev_block_generator=converted_prev_block_generator,
+                    prev_block_validators=converted_prev_block_validators,
+                    prev_block_votes=converted_prev_votes,
+                    is_block_editable=converted_is_block_editable
+                )
 
             if convert_tx_result_to_dict:
                 convert_tx_results = [tx_result.to_dict(to_camel_case) for tx_result in tx_results]
@@ -190,6 +196,8 @@ class IconScoreInnerTask(object):
 
             if next_preps:
                 results["prep"] = next_preps
+            if is_shutdown:
+                results["is_shutdown"] = True
 
             response = MakeResponse.make_response(results)
         except FatalException as e:
@@ -214,7 +222,11 @@ class IconScoreInnerTask(object):
 
     @message_queue_task
     async def query(self, request: dict) -> dict:
-        self._check_icon_service_ready()
+        try:
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
+
         return await self._get_query_response(request)
 
     async def _get_query_response(self, request: dict) -> dict:
@@ -264,7 +276,10 @@ class IconScoreInnerTask(object):
     async def call(self, request: dict):
         Logger.info(tag=_TAG, msg=f'call() start: {request}')
 
-        self._check_icon_service_ready()
+        try:
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
 
         if self._is_thread_flag_on(EnableThreadFlag.QUERY):
             loop = asyncio.get_event_loop()
@@ -296,7 +311,10 @@ class IconScoreInnerTask(object):
 
     @message_queue_task
     async def write_precommit_state(self, request: dict):
-        self._check_icon_service_ready()
+        try:
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
 
         if self._is_thread_flag_on(EnableThreadFlag.INVOKE):
             loop = asyncio.get_event_loop()
@@ -341,8 +359,10 @@ class IconScoreInnerTask(object):
     async def remove_precommit_state(self, request: dict):
         Logger.info(tag=_TAG, msg=f'remove_precommit_state() start')
 
-        self._check_icon_service_ready()
-
+        try:
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
 
         """
         Unused API
@@ -359,7 +379,10 @@ class IconScoreInnerTask(object):
 
         Logger.info(tag=_TAG, msg=f"rollback() start")
 
-        self._check_icon_service_ready()
+        try:
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
 
         if self._is_thread_flag_on(EnableThreadFlag.INVOKE):
             loop = asyncio.get_event_loop()
@@ -398,7 +421,10 @@ class IconScoreInnerTask(object):
 
     @message_queue_task
     async def validate_transaction(self, request: dict):
-        self._check_icon_service_ready()
+        try:
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
 
         if self._is_thread_flag_on(EnableThreadFlag.VALIDATE):
             loop = asyncio.get_event_loop()
@@ -428,9 +454,29 @@ class IconScoreInnerTask(object):
 
     @message_queue_task
     async def change_block_hash(self, _params):
-        self._check_icon_service_ready()
+        try:
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
+
         return ExceptionCode.OK
 
+    @message_queue_task
+    async def dos_guard(self, params: dict) -> dict:
+        try:
+            Logger.info(tag=_TAG, msg=f'dos_guard: params: {params}')
+            self._check_icon_service_ready()
+        except ServiceNotReadyException as e:
+            return MakeResponse.make_error_response(e.code, str(e))
+
+        try:
+            self._icon_service_engine.dos_guard.run(_from=params["from"])
+            response = MakeResponse.make_response(ExceptionCode.OK)
+        except Exception as e:
+            self._log_exception(e, _TAG)
+            response = MakeResponse.make_error_response(ExceptionCode.SYSTEM_ERROR, str(e))
+        return response
+
 
 class MakeResponse:
     @staticmethod
